from .stopword_filter import StopWordFilter
from .textacy_keyterm_extractor import TextacyKeytermExtractor
from .textacy_ngram_extractor import TextacyNgramExtractor
from .textacy_nounchunk_extractor import TextacyNounChunkExtractor
from .textacy_term_extractor import TextacyTermExtractor
from .textacy_triple_extractor import TextacyTripleExtractor



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from spacy.lang.en import English
from textacy import load_spacy_lang


def load_model() -> English:
    """ Load spaCy model consistently

    Returns:
        English: an activated spaCy model
    """
    return load_spacy_lang("en_core_web_sm")



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract Keyterms from Input Text using Textacy """


import pandas as pd
from pandas import DataFrame

from spacy.lang.en import English

from baseblock import Enforcer
from baseblock import Stopwatch
from baseblock import BaseObject

from owl_builder.autotaxo.dmo import TextacyKeytermExtractor
from owl_builder.autotaxo.dmo import TextacyNgramExtractor
from owl_builder.autotaxo.dmo import TextacyNounChunkExtractor
from owl_builder.autotaxo.dmo import TextacyTermExtractor


class ExtractKeyterms(BaseObject):
    """ Aggregation Service for Extracting Keyterms """

    def __init__(self,
                 model: English):
        """
        Created:
            18-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/3
        """
        BaseObject.__init__(self, __name__)
        self._extract_terms = TextacyTermExtractor(model).process
        self._extract_ngrams = TextacyNgramExtractor(model).process
        self._extract_keyterms = TextacyKeytermExtractor(model).process
        self._extract_nounchunks = TextacyNounChunkExtractor(model).process

    def _process(self,
                 input_text: str,
                 use_keyterms: bool,
                 use_ngrams: bool,
                 use_terms: bool,
                 use_nounchunks: bool) -> DataFrame:

        master = []

        if self.isEnabledForDebug:
            Enforcer.is_str(input_text)

        input_text = input_text.replace('-', '')  # hyphens are problematic ...

        if use_keyterms:
            [master.append(x) for x in self._extract_keyterms(input_text)]

        if use_ngrams:
            [master.append(x) for x in self._extract_ngrams(input_text)]

        if use_terms:
            [master.append(x) for x in self._extract_terms(input_text)]

        if use_nounchunks:
            [master.append(x) for x in self._extract_nounchunks(input_text)]

        return pd.DataFrame(master)

    def process(self,
                input_text: str,
                use_terms: bool = True,
                use_keyterms: bool = True,
                use_ngrams: bool = False,
                use_nounchunks: bool = False) -> DataFrame:

        sw = Stopwatch()

        if self.isEnabledForDebug:
            Enforcer.is_str(input_text)

        df = self._process(input_text,
                           use_terms=use_terms,
                           use_keyterms=use_keyterms,
                           use_ngrams=use_ngrams,
                           use_nounchunks=use_nounchunks)

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "Keyword Extraction Complete",
                f"\tTotal Time: {str(sw)}",
                f"\tTotal Size: {len(df)}"]))

        return df



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" OpenAI: Generate Inflection Prompts """


from pprint import pprint
from typing import Callable

from baseblock import (BaseObject, Enforcer, EnvIO, ServiceEventGenerator,
                       Stopwatch)

from owl_builder.autosyns.dmo import OpenAIEventExecutor, OpenAIOutputExtractor
from owl_builder.autosyns.dto import OpenAICache


class GenerateInflectionCall(BaseObject):
    """ OpenAI: Generate Inflection Prompts """

    __generate_event = None
    __execute_event = None
    __extract_output = None

    def __init__(self):
        """
        Created:
            20-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/5
        """
        BaseObject.__init__(self, __name__)
        cache = OpenAICache()
        self._exists_in_cache = cache.exists
        self._write_to_cache = cache.write
        self._read_from_cache = cache.read

    def _generate_event(self) -> Callable:
        if not self.__generate_event:
            self.__generate_event = ServiceEventGenerator().process
        return self.__generate_event

    def _execute_event(self) -> Callable:
        if not self.__execute_event:
            self.__execute_event = OpenAIEventExecutor().process
        return self.__execute_event

    def _extract_output(self) -> Callable:
        if not self.__extract_output:
            self.__extract_output = OpenAIOutputExtractor().process
        return self.__extract_output

    def process(self,
                input_text: str,
                temperature: float = 0.7,
                max_tokens: int = 512,
                top_p: float = 1.0,
                best_of: int = 1,
                frequency_penalty: float = 0.0,
                presence_penalty: float = 0.0) -> dict or None:
        """ Call the OpenAI Text Summarizer

        Args:
            input_text (str): the input text to send to OpenAI
            temperature (float, optional): Controls Randomness. Defaults to 0.7.
                as the value approaches 0.0 the model becomes deterministic and repetitive
            max_tokens (int, optional): the maximum number of tokens to generate. Defaults to 64.
                the max is 2048 and the tokens are shared between input and output
            top_p (float, optional): controls diversity via nucleus sampling. Defaults to 1.0.
            frequency_penalty (float, optional): how much to penalize new tokens based on their existing frequency. Defaults to 0.0.
                decreases the model's likelihood to repeat the same line verbatim
            presence_penalty (float, optional): how much to penalize new tokens based in the text. Defaults to 0.0.
                increases the model's likelihood to talk about new topics

        Returns:
            dict: the complete openAI event
        """

        if self._exists_in_cache(input_text):
            return self._read_from_cache(input_text)

        if not EnvIO.is_true("USE_OPENAI"):
            return None

        sw = Stopwatch()
        if self.isEnabledForDebug:
            Enforcer.is_str(input_text)

        prompt_input = f"""
Generate all the English inflections for a word

Word: troubleshoot
Inflections: troubleshoots,troubleshooting,troubleshooted,troubleshooter,troubleshooters
Word: computer
Inflections: computers, computerized, computerizing, computerize, computerizes, computerizing, computerization, computerizations

Word: {input_text}
Inflections:

        """

        d_openai_input = {
            'input_text': input_text,
            'prompt_input': prompt_input,
            'engine': 'text-davinci-002',
            'temperature': temperature,
            'max_tokens': max_tokens,
            'top_p': top_p,
            'best_of': best_of,
            'frequency_penalty': frequency_penalty,
            'presence_penalty': presence_penalty,
        }

        d_openai_output = self._execute_event()(d_openai_input)
        inflections = self._extract_output()(search_term=input_text,
                                             d_event=d_openai_output)

        # COR-80; Generate an Event Record
        d_event = self._generate_event()(
            service_name=self.component_name(),
            event_name='generate-inflection-call',
            stopwatch=sw,
            data={
                'input_text': input_text,
                'output_text': inflections,
                'openai_input': d_openai_input,
                'openai_output': d_openai_output,
            })

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                f"OpenAI Service Completed ({d_event['service']})",
                f"\tTotal Time: {str(sw)}",
                f"\tInput Text: {input_text.strip()}",
                f"\tOutput Text: {inflections}"]))

        result = {
            'inflections': inflections,
            'events': [d_event]
        }

        self._write_to_cache(data=result,
                             file_name=input_text)
        return result



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Generate an OWL model based on a TTL template """


import os
from datetime import datetime

import pandas as pd
from baseblock import BaseObject, EnvIO, FileIO
from pandas import DataFrame
from rdflib import Graph, Namespace

from owl_builder.autorels.svc import (FindImpliesRelationships,
                                      FindRequiresRelationships,
                                      GenerateRelationshipsTTL)


class GenerateOwlModel(BaseObject):
    """ Generate an OWL model based on a TTL template """

    __authorship = "Generated on TIMESTAMP by AUTHOR"

    def __init__(self):
        """ Change Log:

        Created:
            21-Jul-20922
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/7
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                model_name: str,
                model_path: str,
                model_author: str = None) -> str:
        """ Generate an OWL model

        Args:
            model_name (str): the name of the model
                this will form part of the namespace URI
            model_path (str): the directory the model will be persisted to
            model_author (str, optional): the model author. Defaults to None.

        Returns:
            str: the absolute path to the OWL model
        """

        template_path = FileIO.join_cwd(
            'resources/models/templates/template.owl')
        FileIO.exists_or_error(template_path)
        lines = FileIO.read_lines(template_path)

        ns_default = "http://graffl.ai/template"
        ns_model_name = f"http://graffl.ai/{model_name}"

        def authorship() -> str:
            dt = str(datetime.now())
            if not model_author:
                return f"Generated on {dt}"
            return f"Generated by {model_author} on {dt}"

        normalized = []
        for line in lines:
            if ns_default in line:
                line = line.replace(ns_default, ns_model_name)
            if self.__authorship in line:
                line = line.replace(self.__authorship, authorship())
            normalized.append(line)

        FileIO.exists_or_error(model_path)
        model_file_name = os.path.normpath(
            os.path.join(model_path, f"{model_name}.owl"))
        FileIO.write_lines(normalized, model_file_name)

        return model_file_name



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Orchestrate Synonym Generation for Ontologies """


import pandas as pd
from baseblock import BaseObject
from pandas import DataFrame

from owl_builder.autosyns.svc import GenerateInflectionCall
from owl_builder.autotaxo.bp import AutoTaxoOrchestrator


class AutoSynsOrchestrator(BaseObject):
    """ Orchestrate Synonym Generation for Ontologies """

    def __init__(self):
        """ Change Log:

        Created:
            20-Jul-20922
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/5

        """
        BaseObject.__init__(self, __name__)
        self._find_inflections = GenerateInflectionCall().process

    def _keyterms(self,
                  input_text: str) -> list:
        if ' ' not in input_text:
            return [input_text]

        svc = AutoTaxoOrchestrator()
        keyterms = svc.keyterms(input_text,
                                use_keyterms=True,
                                use_ngrams=True,
                                use_nounchunks=True,
                                use_terms=True)

        unigrams = set()
        for keyterm in keyterms:
            [unigrams.add(x) for x in keyterm.split()]

        return sorted(unigrams)

    def process(self,
                input_text: str) -> list:
        master = []

        for keyterm in self._keyterms(input_text):
            d_result = self._find_inflections(keyterm)
            if d_result:
                csv = ','.join(d_result['inflections'])
                master.append(f"{keyterm}~{csv}")

        return master



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Make an actual OpenAI call """


from pprint import pformat

import openai
from baseblock import BaseObject, CryptoBase, Enforcer, EnvIO


class OpenAIEventExecutor(BaseObject):
    """ Make an actual OpenAI call """

    def __init__(self,
                 timeout: int = 15):
        """ Change Log

        Created:
            20-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/5

        Args:
            timeout (int, optional): the timeout for the API call. Defaults to 15.
        """
        BaseObject.__init__(self, __name__)
        self._openai = self._connect()
        self._timeout = EnvIO.int_or_default(
            'OPENAI_CREATE_TIMEOUT', timeout)  # GRAFFL-380

    def _connect(self):
        key = b'gAAAAABiH-eZKbScaS9reXABcCVeRA-VK7rbh-ZBzH72tfzRjTHIH6y5DmcFPxs1Hbf5suJufyD6Z_WhL4h1N1s_BBGpV5JqGZpVCPoB-dAPIFz6gE3uEgUMv_le5RYej5jnZawccsOSKA1RWWpC-CVTcn80S4LehA=='
        org = b'gAAAAABiH-0FMWAwMybowrACJ6GPkC91E4DgaV2lJoextMR7U4O5DjB_pw9jBUusuCdH9KEUXp-3Iq-Fni1X-eE6ulSg8JEKtZs-bClX_D-2DyvYpv65iPs='

        openai.organization = CryptoBase().decrypt(org)
        openai.api_key = CryptoBase().decrypt(key)

        return openai

    def process(self,
                d_event: dict) -> dict:

        if self.isEnabledForDebug:
            Enforcer.is_dict(d_event)

        if 'best_of' not in d_event:
            d_event['best_of'] = 1

        response = self._openai.Completion.create(
            engine=d_event['engine'],
            prompt=d_event['prompt_input'],
            temperature=d_event['temperature'],
            max_tokens=d_event['max_tokens'],
            top_p=d_event['top_p'],
            best_of=d_event['best_of'],
            frequency_penalty=d_event['frequency_penalty'],
            presence_penalty=d_event['presence_penalty'],
            timeout=self._timeout  # GRAFFL-380
        )

        d_result = dict(response)
        if self.isEnabledForDebug:

            Enforcer.is_dict(d_event)
            self.logger.debug('\n'.join([
                "OpenAI Call Completed",
                pformat(d_result)]))

        return d_result



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Open a file-based model as an rdflib Graph """


from baseblock import BaseObject, FileIO
from rdflib import Graph

from owl_builder.buildr.dmo import OwlGraphConnector


class OpenAsGraph(BaseObject):
    """ Open a file-based model as an rdflib Graph """

    def __init__(self):
        """ Change Log:

        Created:
            21-Jul-20922
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/7
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                model_file_name: str) -> Graph:
        """ Open a file-based model as an rdflib Graph

        Args:
            model_file_name (str): the path to the existing model

        Returns:
            Graph: an instantiation of an rdflib Graph object
        """

        lines = FileIO.read_lines(model_file_name)

        def get_namespace() -> str:
            for line in lines:
                if line.startswith('@prefix'):
                    line = line.split(': <')[-1].split('#')[0]
                    return line

        namespace = get_namespace()
        prefix = namespace.split('/')[-1].strip()

        graph = OwlGraphConnector(
            prefix=prefix,
            namespace=namespace,
            model_file_name=model_file_name).graph()

        return graph



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract Keyterms using an OpenAI Model """


from typing import List

from baseblock import Enforcer
from baseblock import Stopwatch
from baseblock import BaseObject

from openai_helper import chat


class ExtractKeytermsOpenAI(BaseObject):
    """ Extract Keyterms using an OpenAI Model """

    def __init__(self):
        """
        Created:
            28-Mar-2023
            craigtrim@gmail.com
            *   https://github.com/craigtrim/owl-builder/issues/5
        """
        BaseObject.__init__(self, __name__)

    @staticmethod
    def _prompt(input_text: str) -> str:
        return f"Some text is provided below. Given the text, extract up to 10 keywords from the text. Avoid stopwords.---------------------\\n{input_text}\\n---------------------\\nProvide keywords in the following comma-separated format: \'KEYWORDS: <keywords>\'\\n"

    def process(self,
                input_text: str) -> List[str]:

        sw = Stopwatch()

        if self.isEnabledForDebug:
            Enforcer.is_str(input_text)

        keyterms = chat(
            input_prompt=self._prompt(input_text),
            messages=None,
            remove_emojis=True)

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "Keyword Extraction Complete",
                f"\tTotal Time: {str(sw)}",
                f"\tExtracted Terms: {keyterms}"]))

        return keyterms



import os

from baseblock import Enforcer

from owl_builder.autosyns.bp import AutoSynsOrchestrator

os.environ['USE_OPENAI'] = str(False)


def test_orchestrator_1():
    input_text = """
        A local area network (LAN) is a computer network that interconnects computers within a limited area such as a residence, school, laboratory, university campus or office building.
        By contrast, a wide area network (WAN) not only covers a larger geographic distance, but also generally involves leased telecommunication circuits.
        Ethernet and Wi-Fi are the two most common technologies in use for local area networks.
        Historical network technologies include ARCNET, Token Ring, and AppleTalk.
    """.strip()

    bp = AutoSynsOrchestrator()
    assert bp

    result = bp.process(input_text)

    if result:
        [print(x) for x in result]
        Enforcer.is_list_of_str(result)


def test_orchestrator_2():
    input_text = "network"

    bp = AutoSynsOrchestrator()
    assert bp

    result = bp.process(input_text)

    if result:
        [print(x) for x in result]
        Enforcer.is_list_of_str(result)


def main():
    os.environ['USE_OPENAI'] = str(True)
    test_orchestrator_1()
    test_orchestrator_2()


if __name__ == "__main__":
    main()



from baseblock import Enforcer
from tabulate import tabulate

from owl_builder.autotaxo.dto import load_model
from owl_builder.autotaxo.svc import (ExtractKeyterms, FilterKeyterms,
                                      GenerateTaxonomyDataFrame,
                                      GenerateTaxonomyTTL)

input_text = """
    A local area network (LAN) is a computer network that interconnects computers within a limited area such as a residence, school, laboratory, university campus or office building.
    By contrast, a wide area network (WAN) not only covers a larger geographic distance, but also generally involves leased telecommunication circuits.
    Ethernet and Wi-Fi are the two most common technologies in use for local area networks.
    Historical network technologies include ARCNET, Token Ring, and AppleTalk.
""".strip()


def test_service():

    model = load_model()

    extract = ExtractKeyterms(model).process
    assert extract

    df = extract(input_text)
    print(tabulate(df, headers='keys', tablefmt='psql'))

    filter = FilterKeyterms().process
    assert filter

    terms = filter(df)
    print(terms)

    gendf = GenerateTaxonomyDataFrame().process
    assert gendf

    df = gendf(terms)
    print(tabulate(df, headers='keys', tablefmt='psql'))

    ttlgen = GenerateTaxonomyTTL().process
    assert ttlgen

    results = ttlgen(df)
    [print(x) for x in results]
    Enforcer.is_list_of_str(results)


def main():
    test_service()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract Ngrams from Input Text using Textacy """


from baseblock import BaseObject, Stopwatch
from spacy.lang.en import English
from textacy import make_spacy_doc
from textacy.extract import ngrams

from owl_builder.autotaxo.dmo import StopWordFilter


class TextacyNgramExtractor(BaseObject):
    """ Extract Ngrams from Input Text using Textacy

    Documentation:
        https://textacy.readthedocs.io/en/latest/api_reference/extract.html#textacy.extract.basics.ngrams
    """

    def __init__(self,
                 model: English):
        """ Change Log

        Created:
            20-Apr-2022
            craigtrim@gmail.com
            *   in pursuit of
                https://github.com/grafflr/graffl-core/issues/302
        Updated:
            18-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/3

        Args:
            model (English): spaCy model
        """
        BaseObject.__init__(self, __name__)
        self._model = model
        self._has_stopword = StopWordFilter().has_stopword

    def _process(self,
                 input_text: str,
                 ngram_level: int,
                 filter_stops: bool,
                 filter_punct: bool,
                 filter_nums: bool,
                 case_sensitive: bool,
                 term_frequency: int) -> list:

        if not case_sensitive:
            input_text = input_text.lower()

        doc = make_spacy_doc(input_text, lang=self._model)

        results = list(ngrams(doc, ngram_level,
                              include_pos=('NOUN', 'PROPN', 'ADJ'),
                              exclude_pos=('ADV'),
                              filter_stops=filter_stops,
                              filter_punct=filter_punct,
                              filter_nums=filter_nums))

        from collections import Counter
        c = Counter()
        for result in results:
            c.update({result: 1})

        s = set()
        for term in c:
            frequency = c[term]
            if frequency >= term_frequency:
                s.add(term)

        return sorted(s)

    def process(self,
                input_text: str,
                ngram_level: int = 3,
                term_frequency: int = 1,
                filter_stops: bool = True,
                filter_punct: bool = True,
                filter_nums: bool = True,
                case_sensitive: bool = False) -> list:
        """ Extract n-Grams from Input Text

        Args:
            input_text (str): input text of any length
            ngram_level (int, optional): n-Gram level to extract. Defaults to 3.
            filter_stops (bool, optional): Filter Stopwords. Defaults to True.
            filter_punct (bool, optional): Filter Punctuation. Defaults to True.
            filter_nums (bool, optional): Remove Numbers from results. Defaults to True.
            term_frequency (int, optional): Number of times a term must repeat. Defaults to 2.
            case_sensitive (bool, optional): Determine if input text case should be maintained.  Defaults to False

        Returns:
            list: n-Gram results
        """

        sw = Stopwatch()

        results = self._process(input_text=input_text,
                                ngram_level=ngram_level,
                                filter_stops=filter_stops,
                                filter_punct=filter_punct,
                                filter_nums=filter_nums,
                                term_frequency=term_frequency,
                                case_sensitive=case_sensitive)

        # the result-set is a list of spacy.tokens.span.Span elements
        results = [x.text for x in results]

        if filter_stops:
            results = [x for x in results if not self._has_stopword(x)]

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "N-Gram Extraction Complete",
                f"\tTotal Time: {str(sw)}",
                f"\tTotal Size: {len(results)}"]))

        master = []

        for result in results:
            master.append({
                "Term": result,
                "Size": ngram_level,
                "Score": None,
                "Source": "Ngram"
            })

        return master



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Create "Implies" Relationships between entities """


from baseblock import BaseObject


class FindImpliesRelationships(BaseObject):
    """ Create "Implies" Relationships between entities """

    def __init__(self):
        """ Change Log:

        Created:
            20-Jul-20922
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/4

        """
        BaseObject.__init__(self, __name__)

    def process(self,
                terms: list) -> list or None:
        """ Find 'Implies' Relationships between terms

        Sample Input:
            ['network hardware', 'network protocol', 'network', 'protocol']

        Sample Output (pseudo):
            [   "network hardware" implies "network",
                "network protocol" implies "protocol"
            ]

        Args:
            terms (list): a list of plain text terms

        Returns:
            list or None: a list of S/P/O triple results
        """

        # normalize incoming terms
        # the function expects 'alpha_beta_gamma' formatted terms
        terms = [x.replace(' ', '_') for x in terms]

        unigrams = [x for x in terms if '_' not in x]
        ngrams = [x for x in terms if '_' in x]

        def match(input_text: str) -> str or None:
            l_text = f"_{input_text}"
            r_text = f"{input_text}_"
            for ngram in ngrams:
                if l_text in ngram:
                    return ngram, ngram.replace(l_text, '')
                if r_text in ngram:
                    return ngram, ngram.replace(r_text, '')
            return None, None

        results = []
        for unigram in unigrams:
            x, y = match(unigram)
            if x and y:
                results.append({
                    "Subject": x,
                    "Predicate": "implies",
                    "Object": y
                })

        return results



from owl_builder.autotaxo.dmo import StopWordFilter


def test_component():

    dmo = StopWordFilter()
    assert dmo

    assert dmo.has_stopword('network that interconnects')
    assert not dmo.has_stopword('leased telecommunication circuits')



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Generate Taxonomy Suitable for use in an OWL file """


import pandas as pd

from pandas import DataFrame

from baseblock import Stopwatch
from baseblock import BaseObject


class GenerateTaxonomyDataFrame(BaseObject):
    """ Generate Taxonomy Suitable for use in an OWL file """

    def __init__(self):
        """ Change Log

        Created:
            16-Apr-2022
            craigtrim@gmail.com
            *   refactored out of jupyter notebook:
                    GRAFFL-286 Textacy Textrank
                    http://localhost:8888/notebooks/grafflbox/GRAFFL-286%20Textacy%20Textrank.ipynb
                https://github.com/grafflr/graffl-core/issues/286
        Updated:
            14-Dec-2022
            craigtrim@gmail.com
            *   fix confidence defect
                https://github.com/craigtrim/owl-builder/issues/3
        """
        BaseObject.__init__(self, __name__)

    def _decompose_term(self,
                        term: str) -> list:
        taxonomy = []

        tokens = term.split()
        if len(tokens) == 1:
            return [term]

        for i in range(len(tokens)):
            current = tokens[:i]
            if len(current):
                current.reverse()
                taxonomy.append(' '.join(current))

        return taxonomy

    def _process(self,
                 results: list) -> list:
        master = []

        for result in results:

            tokens = result.split()
            tokens.reverse()

            taxonomy = []
            for i in range(len(tokens)):
                current = tokens[:i]
                if len(current):
                    current.reverse()
                    taxonomy.append(' '.join(current))

            if result not in taxonomy:
                taxonomy.append(result)

            if not len(taxonomy):
                continue

            for i in range(len(taxonomy)):
                if i + 1 < len(taxonomy):
                    master.append({
                        "Parent": taxonomy[i],
                        "Child": taxonomy[i + 1],
                        "Confidence": None  # issues-3-1352031348
                    })

        return master

    def process(self,
                results: list) -> DataFrame:

        sw = Stopwatch()

        master = self._process(results)

        df = pd.DataFrame(master)

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "Taxonomy Generation Complete",
                f"\tTotal Time: {str(sw)}",
                f"\tTotal Size: {len(df)}"]))

        return df



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Generate TTL Suitable for use in an OWL file """


from baseblock import BaseObject
from pandas import DataFrame


class GenerateRelationshipsTTL(BaseObject):
    """ Generate TTL Suitable for use in an OWL file """

    def __init__(self):
        """
        Created:
            20-Jul-20922
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/4
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                df: DataFrame) -> list:

        def to_entity(some_value: str) -> str:
            return some_value.replace(' ', '_').lower().strip()

        def to_label(some_value: str) -> str:
            tokens = some_value.replace('_', ' ').split(' ')
            tokens = [f"{x[0].upper()}{x[1:]}" for x in tokens]
            return ' '.join(tokens).strip()

        def ttl_subject() -> str:
            return """
            ###  http://graffl.ai/pathology#EntityName
            :EntityName rdf:type owl:Class ;
            rdfs:label "EntityLabel" ;
            :PredicateName :ObjectName .
            """.strip()

        def ttl_object() -> str:
            return """
            ###  http://graffl.ai/pathology#EntityName
            :EntityName rdf:type owl:Class ;
            rdfs:label "EntityLabel" .
            """.strip()

        snippets = []
        for _, row in df.iterrows():

            subject_entity = to_entity(row['Subject'])
            subject_label = to_label(row['Subject'])

            object_entity = to_entity(row['Object'])
            object_label = to_label(row['Object'])

            predicate = row['Predicate']

            ttl_s = ttl_subject()
            ttl_s = ttl_s.replace('EntityName', subject_entity)
            ttl_s = ttl_s.replace('EntityLabel', subject_label)
            ttl_s = ttl_s.replace('PredicateName', predicate)
            ttl_s = ttl_s.replace('ObjectName', object_entity)
            snippets.append(ttl_s)

            ttl_o = ttl_object()
            ttl_o = ttl_o.replace('EntityName', object_entity)
            ttl_o = ttl_o.replace('EntityLabel', object_label)
            snippets.append(ttl_o)

        return snippets



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract Ngrams from Input Text using Textacy """

from collections import Counter

from baseblock import BaseObject, Stopwatch
from spacy.lang.en import English
from textacy import make_spacy_doc
from textacy.extract import noun_chunks

from owl_builder.autotaxo.dmo import StopWordFilter


class TextacyNounChunkExtractor(BaseObject):
    """ Extract Ngrams from Input Text using Textacy

    Documentation:
        https://textacy.readthedocs.io/en/latest/api_reference/extract.html#textacy.extract.basics.noun_chunks
    """

    def __init__(self,
                 model: English):
        """ Change Log

        Created:
            15-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/2
        Updated:
            18-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/3
        Args:
            model (English): spaCy model
        """
        BaseObject.__init__(self, __name__)
        self._model = model
        self._has_stopword = StopWordFilter().has_stopword

    def _process(self,
                 input_text: str,
                 drop_determiners: bool,
                 case_sensitive: bool,
                 min_freq: int) -> list:

        if not case_sensitive:
            input_text = input_text.lower()

        doc = make_spacy_doc(input_text, lang=self._model)

        results = list(noun_chunks(doc,
                                   min_freq=min_freq,
                                   drop_determiners=drop_determiners))

        c = Counter()
        for result in results:
            c.update({result: 1})

        s = set()
        for term in c:
            frequency = c[term]
            if frequency >= min_freq:
                s.add(term)

        return sorted(s)

    def process(self,
                input_text: str,
                min_freq: int = 1,
                filter_stops: bool = True,
                drop_determiners: bool = True,
                case_sensitive: bool = False) -> list:
        """ Extract n-Grams from Input Text

        Args:
            input_text (str): input text of any length
            min_freq (int, optional): Number of times a term must repeat. Defaults to 3.
            filter_stops (bool, optional): Filter Stopwords. Defaults to True.
            drop_determiners (bool, optional): drop determiners from the start of noun chunks. Defaults to True.
            case_sensitive (bool, optional): Determine if input text case should be maintained.  Defaults to False
            as_dataframe (bool, optional): Returns the result set as a pandas Dataframe.  Defaults to False

        Returns:
            list: n-Gram results
        """

        sw = Stopwatch()

        results = self._process(input_text=input_text,
                                drop_determiners=drop_determiners,
                                case_sensitive=case_sensitive,
                                min_freq=min_freq)

        # the result-set is a list of spacy.tokens.span.Span elements
        results = [x.text.strip() for x in results]

        if filter_stops:
            results = [x for x in results if not self._has_stopword(x)]

        # I consider a 'noun chunk' to be 2..* nouns
        # results = [x for x in results if ' ' in x]

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "Noun Chunk Extraction Complete",
                f"\tTotal Time: {str(sw)}",
                f"\tTotal Size: {len(results)}"]))

        master = []

        for result in results:
            master.append({
                "Term": result,
                "Size": len(result.split()),
                "Source": "NounChunk",
                "Score": None,
            })

        return master



from baseblock import Enforcer

from owl_builder.autorels.svc import FindRequiresRelationships


def test_service():
    svc = FindRequiresRelationships()
    assert svc

    input_terms = [
        'network hardware',
        'network protocol',
        'network',
        'protocol'
    ]

    actual_results = svc.process(input_terms)
    Enforcer.is_list_of_dicts(actual_results)


def main():
    test_service()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Orchestrate Taxonomy Generation """


import os

from rdflib import Graph

from owl_builder.buildr.bp import OwlBuilderAPI


def test_owl_buildr_api():
    api = OwlBuilderAPI()

    model_file_name = api.generate(
        model_name="mytest001",
        model_path=os.getcwd(),
        model_author="ctrim")

    print(f"Generated Test Model: {model_file_name}")
    assert os.path.exists(model_file_name)

    version = api.add_entities(
        ttl_entities=[],
        model_file_name=model_file_name)

    assert version == "0.1.1"
    assert os.path.exists(model_file_name)

    version = api.add_entities(
        ttl_entities=[],
        model_file_name=model_file_name)

    assert version == "0.1.2"
    assert os.path.exists(model_file_name)

    version = api.add_entities(
        ttl_entities=[],
        model_file_name=model_file_name)

    assert version == "0.1.3"
    assert os.path.exists(model_file_name)

    version = api.add_entities(
        ttl_entities=[],
        model_file_name=model_file_name)

    assert version == "0.1.4"
    assert os.path.exists(model_file_name)

    g = api.to_graph(model_file_name)
    assert g
    assert type(g) == Graph

    os.remove(model_file_name)
    assert not os.path.exists(model_file_name)


def main():
    test_owl_buildr_api()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

from typing import List, Optional

from pandas import DataFrame

from .autorels import *
from .autosyns import *
from .autotaxo import *
from .autotaxo.bp import AutoTaxoOrchestrator

autotaxo = AutoTaxoOrchestrator()


def build_ttl(input: str or DataFrame) -> Optional[List[str]]:
    """ Generate TTL Results for augmenatation of OWL model

    Args:
        input (str or DataFrame): either input text or a DataFrame
            input_text:     sent to dataframe(...) method
                Sample Input:
                    A local area network (LAN) is a computer network that interconnects computers within a limited area.
            DataFrame:      must have output format equivalent to dataframe(...) method
                Sample Input:
                    +----+---------------------------+----------------------------------+
                    |    | Parent                    | Child                            |
                    |----+---------------------------+----------------------------------+
                    |  0 | circuit                   | telecommunication circuit        |
                    |  1 | telecommunication circuit | leased telecommunication circuit |
                    |  2 | technology                | network technology               |
                    +----+---------------------------+----------------------------------+

    Sample Output:
        ###  http://graffl.ai/pathology#telecommunication_circuit
                    :telecommunication_circuit rdf:type owl:Class ;
                    rdfs:label "Telecommunication Circuit" ;
                    rdfs:subClassOf :circuit .
        ###  http://graffl.ai/pathology#circuit
                    :circuit rdf:type owl:Class ;
                    rdfs:label "Circuit" .
        ###  http://graffl.ai/pathology#leased_telecommunication_circuit
                    :leased_telecommunication_circuit rdf:type owl:Class ;
                    rdfs:label "Leased Telecommunication Circuit" ;
                    rdfs:subClassOf :telecommunication_circuit .
        ...
        ###  http://graffl.ai/pathology#token_ring
                    :token_ring rdf:type owl:Class ;
                    rdfs:label "Token Ring" ;
                    rdfs:subClassOf :ring .
        ###  http://graffl.ai/pathology#ring
                    :ring rdf:type owl:Class ;
                    rdfs:label "Ring" .

    Returns:
        Optional[List[str]]: TTL results for OWL model
    """
    return autotaxo.build_ttl(input)


def keyterms_openai(input_text: str) -> Optional[List[str]]:
    pass

def keyterms(input_text: str,
             use_terms: bool = True,
             use_keyterms: bool = True,
             use_ngrams: bool = False,
             use_nounchunks: bool = False) -> Optional[List[str]]:
    """ Generate KeyTerms as a simple list

    Args:
        input_text (str): input text of any length or description
        use_terms (bool, optional). Use Simple Term extraction algorithms. Default is True.
        use_keyterms (bool, optional). Use KeyTerm extraction algorithms. Default is True.
        use_ngrams (bool, optional). Use n-Gram extraction algorithms. Default is False.
        use_nounchunks (bool, optional). Use Noun Chunk extraction algorithms. Default is False.

    Sample Input:
        A local area network (LAN) is a computer network that interconnects computers within a limited area such as a residence, school, laboratory, university campus or office building.
        By contrast, a wide area network (WAN) not only covers a larger geographic distance, but also generally involves leased telecommunication circuits.
        Ethernet and Wi-Fi are the two most common technologies in use for local area networks.
        Historical network technologies include ARCNET, Token Ring, and AppleTalk.

    Sample Output:
        [   'leased telecommunication circuit',
            'historical network technology',
            'large geographic distance',
            'interconnects computer',
            'local area network',
            'university campus',
            'common technology',
            'wide area network',
            'computer network',
            'office building',
            'include arcnet',
            'limited area',
            'token ring'
        ]

    Returns:
        Optional[List[str]]: list of keyterms
    """

    return autotaxo.keyterms(
        input_text=input_text,
        use_terms=use_terms,
        use_keyterms=use_keyterms,
        use_ngrams=use_ngrams,
        use_nounchunks=use_nounchunks)



from .extract_keyterms import ExtractKeyterms
from .filter_keyterms import FilterKeyterms
from .generate_taxonomy_dataframe import GenerateTaxonomyDataFrame
from .generate_taxonomy_ttl import GenerateTaxonomyTTL
from .extract_keyterms_openai import ExtractKeytermsOpenAI



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Connect to an RDF Graph (Ontology) """


from baseblock import BaseObject
from rdflib import Graph, Namespace


class OwlGraphConnector(BaseObject):
    """ Connect to an RDF Graph (Ontology) """

    def __init__(self,
                 prefix: str,
                 namespace: str,
                 model_file_name: str):
        """ Change Log

        Created:
            21-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/7

        Args:
            prefix (str): the query prefix
            namespace (str): the ontology namespace
            model_file_name (str): the absolute path to the model
        """
        BaseObject.__init__(self, __name__)

        self._format = "ttl"
        self._prefix = prefix
        self._namespace = namespace
        self._model_file_name = model_file_name

        self._graph = self._process()

        self.logger.debug('\n'.join([
            "Loading Ontology",
            f"\tPath: {self._model_file_name}",
            f"\tNamespace: {self._namespace}",
            f"\tPrefix: {self._prefix}",
            f"\tFormat: {self._format}"]))

    def _process(self) -> Graph:
        """load the ontology from disk as an RDF Graph

        Returns:
            Graph: an instantiated and in-memory RDF Graph
        """
        g = Graph()

        g.parse(self._model_file_name,
                format=self._format)

        g.bind(self._prefix,
               Namespace(self._namespace))

        return g

    def graph(self) -> Graph:
        return self._graph



from owl_builder.autotaxo.dmo import TextacyKeytermExtractor
from owl_builder.autotaxo.dto import load_model

model = load_model()


def test_component():

    input_text = """
        A local area network (LAN) is a computer network that interconnects computers within a limited area such as a residence, school, laboratory, university campus or office building.
        By contrast, a wide area network (WAN) not only covers a larger geographic distance, but also generally involves leased telecommunication circuits.
        Ethernet and Wi-Fi are the two most common technologies in use for local area networks.
        Historical network technologies include ARCNET, Token Ring, and AppleTalk.
    """.strip()

    dmo = TextacyKeytermExtractor(model)
    assert dmo

    results = dmo.process(top_n=15,
                          input_text=input_text)

    print(results)


def main():
    test_component()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Generate Taxonomy Suitable for use in an OWL file """


from collections import defaultdict

from baseblock import BaseObject
from pandas import DataFrame


class GenerateTaxonomyTTL(BaseObject):
    """ Generate Taxonomy Suitable for use in an OWL file """

    def __init__(self):
        """ Change Log

        Created:
            16-Apr-2022
            craigtrim@gmail.com
            *   refactored out of jupyter notebook:
                    GRAFFL-286 Textacy Textrank
                    http://localhost:8888/notebooks/grafflbox/GRAFFL-286%20Textacy%20Textrank.ipynb
                https://github.com/grafflr/graffl-core/issues/286
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                df: DataFrame) -> list:

        def to_entity(some_value: str) -> str:
            return some_value.replace(' ', '_').lower().strip()

        def to_label(some_value: str) -> str:
            tokens = some_value.split(' ')
            tokens = [f"{x[0].upper()}{x[1:]}" for x in tokens]
            return ' '.join(tokens).strip()

        def template_1() -> str:
            return """
            ###  http://graffl.ai/pathology#ChildEntity
            :ChildEntity rdf:type owl:Class ;
            rdfs:label "ChildLabel" ;
            rdfs:subClassOf :ParentEntity .
            """.strip()

        def template_2() -> str:
            return """
            ###  http://graffl.ai/pathology#ChildEntity
            :ChildEntity rdf:type owl:Class ;
            rdfs:label "ChildLabel" .
            """.strip()

        def is_valid(value: str) -> bool:
            temp = value.replace(' ', '').replace('_', '').strip()
            total_chars = sum([x.isalpha() for x in temp])
            return len(temp) == total_chars

        snippets = []

        d_sorted = defaultdict(set)

        # putting the DataFrame into a dictionary helps ensure sort order
        # testing is easier if we can guarantee entity generation order
        for _, row in df.iterrows():
            d_sorted[row['Parent']].add(row['Child'])

        parents = [x for x in d_sorted if is_valid(x)]
        for parent in parents:

            parent_entity = to_entity(parent)
            parent_label = to_label(parent)

            children = [x for x in sorted(d_sorted[parent]) if is_valid(x)]
            for child in children:

                child_entity = to_entity(child)
                child_label = to_label(child)

                snippets.append(template_1().replace('ParentEntity', parent_entity).replace(
                    'ChildEntity', child_entity).replace('ChildLabel', child_label))

                snippets.append(template_2().replace(
                    'ChildEntity', parent_entity).replace('ChildLabel', parent_label))

        return snippets



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract Output from OpenAI Response """


from pprint import pformat

from baseblock import BaseObject


class OpenAIOutputExtractor(BaseObject):
    """ Extract Output from OpenAI Response """

    def __init__(self):
        """
        Created:
            20-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/5
        """
        BaseObject.__init__(self, __name__)

    def process(self,
                d_event: dict) -> dict:
        """ Like all OpenAI events there are multiple possible format outcomes

        The optimal outcome is
            {
                'choices': [
                    {
                        'finish_reason': 'stop',
                        'index': 0,
                        'logprobs': None,
                        'text': ' Technologies, technologized, technologizing, technologize, technologizes, technologizing, technologization, technologizations'
                    }
                ]
            }

        Args:
            d_event (dict): _description_

        Returns:
            dict: _description_
        """

        def has_expected_format() -> bool:
            if 'choices' not in d_event:
                return False
            if not len(d_event['choices']):
                return False
            if 'text' not in d_event['choices'][0]:
                return False
            return True

        if not has_expected_format():
            self.logger.warning('\n'.join([
                "Unexpected OpenAI Outcome",
                pformat(d_event)]))
            return None

        output_text = d_event['choices'][0]['text']

        inflections = [x.strip().lower()
                       for x in output_text.split(',')]

        inflections = sorted(set(inflections), key=len, reverse=True)

        return inflections



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Simple Stopword Filter """


from baseblock import BaseObject

from owl_builder.autotaxo.dto import stopwords


class StopWordFilter(BaseObject):
    """ Simple Stopword Filter """

    __starting_filter = [
        'generally',
        'involves',
    ]

    __ending_filter = [
        'include',
    ]

    def __init__(self):
        """
        Created:
            20-Apr-2022
            craigtrim@gmail.com
            *   in pursuit of
                https://github.com/grafflr/graffl-core/issues/302
        """
        BaseObject.__init__(self, __name__)

    def has_stopword(self,
                     input_text: str) -> bool:
        """ Custom Layer for removing Stopwords beyond OTB capabilities

        Args:
            input_text (str): an incoming string of any length

        Returns:
            bool: True if the input text contains a stop word
        """

        input_text = input_text.lower().strip()
        if input_text in stopwords:
            return True

        tokens = input_text.split()
        if len(tokens) == 1:
            return False

        if tokens[0] in self.__starting_filter:
            return True

        if tokens[-1] in self.__ending_filter:
            return True

        for token in tokens:
            if token in stopwords:
                return True

        return False



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Orchestrate Taxonomy Generation """


import pandas as pd
from baseblock import BaseObject
from pandas import DataFrame

from owl_builder.autorels.svc import (FindImpliesRelationships,
                                      FindRequiresRelationships,
                                      GenerateRelationshipsTTL)


class AutoRelsOrchestrator(BaseObject):
    """ Orchestrate Taxonomy Generation """

    def __init__(self):
        """ Change Log:

        Created:
            18-Jul-20922
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/4

        """
        BaseObject.__init__(self, __name__)

    def dataframe(self,
                  terms: list) -> DataFrame or None:

        master = []

        results = FindImpliesRelationships().process(terms)
        if results:
            [master.append(x) for x in results]

        results = FindRequiresRelationships().process(terms)
        if results:
            [master.append(x) for x in results]

        if not master:
            return None

        return pd.DataFrame(master)

    def ttl(self,
            df: DataFrame) -> list:
        return GenerateRelationshipsTTL().process(df)



from baseblock import Enforcer

from owl_builder.autotaxo.bp import AutoTaxoOrchestrator


def issues_08():
    # https://github.com/craigtrim/buildowl/issues/8

    bp = AutoTaxoOrchestrator()
    assert bp

    result = bp.keyterms("show my history")
    print(result)

    Enforcer.is_list_of_str(result)


def main():
    issues_08()


if __name__ == "__main__":
    main()



from owl_builder.autotaxo.bp import AutoTaxoOrchestrator


def test_01():
    input_text = """The best emoji for this sentence is the smiley face with sunglasses emoji.
                    This emoji conveys that Reno is a fun place to visit,
                    and that there are many activities to enjoy while you're there.!""".strip()

    bp = AutoTaxoOrchestrator()
    assert bp

    results = bp.keyterms(input_text)
    print(results)


def test_02():
    input_text = """tell me:""".strip()

    bp = AutoTaxoOrchestrator()
    assert bp

    results = bp.keyterms(input_text)
    print(results)


def test_03():
    input_text = """""".strip()

    bp = AutoTaxoOrchestrator()
    assert bp

    results = bp.keyterms(input_text)
    print(results)


def main():
    test_01()
    test_02()
    test_03()


if __name__ == "__main__":
    main()



from baseblock import Enforcer
from pandas import DataFrame
from tabulate import tabulate

from owl_builder.autorels.bp import AutoRelsOrchestrator


def test_orchestrator():
    bp = AutoRelsOrchestrator()
    assert bp

    input_terms = [
        'network hardware',
        'network protocol',
        'network',
        'protocol'
    ]

    df_results = bp.dataframe(input_terms)
    assert df_results is not None
    assert type(df_results) == DataFrame

    print(tabulate(df_results, headers='keys', tablefmt='psql'))

    ttl_results = bp.ttl(df_results)
    Enforcer.is_list_of_str(ttl_results)

    [print(x) for x in ttl_results]


def main():
    test_orchestrator()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract Ngrams from Input Text using Textacy """


from functools import partial

from baseblock import BaseObject
from spacy.lang.en import English
from textacy import make_spacy_doc
from textacy.extract import entities, ngrams, terms

from owl_builder.autotaxo.dmo import StopWordFilter
from owl_builder.autotaxo.dto import load_model


class TextacyTermExtractor(BaseObject):
    """ Extract Terms from Input Text using Textacy

    Documentation:
        https://textacy.readthedocs.io/en/latest/api_reference/extract.html#textacy.extract.basics.terms
    """

    def __init__(self,
                 model: English = None):
        """ Change Log

        Created:
            18-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/3
        Args:
            model (English): spaCy model
        """
        BaseObject.__init__(self, __name__)
        if model is None:
            model = load_model()

        self._model = model
        self._has_stopword = StopWordFilter().has_stopword

    def _process(self,
                 input_text: str,
                 case_sensitive: bool) -> list:

        if not case_sensitive:
            input_text = input_text.lower()

        doc = make_spacy_doc(input_text, lang=self._model)

        master = []

        for result in terms(doc, ngs=2, ents=True, ncs=True):
            master.append({
                "Term": result.text,
                "Size": len(result.text.split()),
                "Score": None,
                "Source": "terms.1",
            })

        for result in terms(doc, ngs=lambda doc: ngrams(doc, n=2)):
            master.append({
                "Term": result.text,
                "Size": len(result.text.split()),
                "Score": None,
                "Source": "terms.2",
            })

        for result in terms(doc, ents=entities):
            master.append({
                "Term": result.text,
                "Size": len(result.text.split()),
                "Score": None,
                "Source": "terms.3",
            })

        for result in terms(doc, ents=partial(entities, include_types="PERSON")):
            master.append({
                "Term": result.text,
                "Size": len(result.text.split()),
                "Score": None,
                "Source": "terms.4",
            })

        return master

    def process(self,
                input_text: str,
                filter_stops: bool = True,
                case_sensitive: bool = False) -> list:
        """ Extract n-Grams from Input Text

        Args:
            input_text (str): input text of any length
            min_freq (int, optional): Number of times a term must repeat. Defaults to 3.
            filter_stops (bool, optional): Filter Stopwords. Defaults to True.
            drop_determiners (bool, optional): drop determiners from the start of noun chunks. Defaults to True.
            case_sensitive (bool, optional): Determine if input text case should be maintained.  Defaults to False
            as_dataframe (bool, optional): Returns the result set as a pandas Dataframe.  Defaults to False

        Returns:
            list: n-Gram results
        """

        results = self._process(input_text=input_text,
                                case_sensitive=case_sensitive)

        if filter_stops:
            results = [x for x in results if not self._has_stopword(x['Term'])]

        return results



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract Keyterms from Input Text using Textacy """


from statistics import StatisticsError

from baseblock import BaseObject, Stopwatch
from spacy.lang.en import English
from textacy import make_spacy_doc
from textacy.extract import keyterms as kt


class TextacyKeytermExtractor(BaseObject):
    """ Extract Keyterms from Input Text using Textacy

    Documentation:
        https://textacy.readthedocs.io/en/latest/api_reference/extract.html#keyterms
    """

    def __init__(self,
                 model: English):
        """ Change Log

        Created:
            20-Apr-2022
            craigtrim@gmail.com
            *   https://github.com/grafflr/graffl-core/issues/302
        Updated:
            18-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/3
        Updated:
            8-Aug-2022
            craigtrim@gmail.com
            *   fix small input error
                https://github.com/craigtrim/buildowl/issues/8
        Updated:
            10-Jan-2023
            craigtrim@gmail.com
            *   textrank algorithms require networkx=2.8.8; no code changes were required in this module
                https://github.com/craigtrim/owl-builder/issues/4

        Args:
            model (English): spaCy model
        """
        BaseObject.__init__(self, __name__)
        self._model = model

    def _process(self,
                 top_n: int,
                 input_text: str) -> list:

        master = []

        doc = make_spacy_doc(input_text, lang=self._model)

        # TextRank: window_size=2, edge_weighting="binary", position_bias=False
        for result in kt.textrank(doc,
                                  topn=top_n,
                                  window_size=2,
                                  normalize="lemma",
                                  edge_weighting='binary',
                                  position_bias=False):
            master.append({
                "Term": result[0],
                "Size": len(result[0].split()),
                "Score": result[1],
                "Source": "kt.textrank",
            })

        # SingleRank: window_size=10, edge_weighting="count", position_bias=False
        for result in kt.textrank(doc,
                                  topn=top_n,
                                  window_size=10,
                                  normalize="lemma",
                                  edge_weighting='count',
                                  position_bias=False):
            master.append({
                "Term": result[0],
                "Size": len(result[0].split()),
                "Score": result[1],
                "Source": "kt.singlerank",
            })

        # PositionRank: window_size=10, edge_weighting="count", position_bias=True
        for result in kt.textrank(doc,
                                  topn=top_n,
                                  window_size=10,
                                  normalize="lemma",
                                  edge_weighting='count',
                                  position_bias=True):
            master.append({
                "Term": result[0],
                "Size": len(result[0].split()),
                "Score": result[1],
                "Source": "kt.positionrank",
            })

        # yake algorithm
        try:
            for result in kt.yake(doc,
                                  topn=top_n,
                                  window_size=10,
                                  normalize="lemma"):
                master.append({
                    "Term": result[0],
                    "Size": len(result[0].split()),
                    "Score": result[1],
                    "Source": "yake",
                })

            # scake algorithm
            for result in kt.scake(doc,
                                   topn=top_n,
                                   normalize="lemma"):
                master.append({
                    "Term": result[0],
                    "Size": len(result[0].split()),
                    "Score": result[1],
                    "Source": "scake",
                })

            # sgrank algorithm
            for result in kt.sgrank(doc,
                                    topn=top_n,
                                    window_size=10,
                                    normalize="lemma"):
                master.append({
                    "Term": result[0],
                    "Size": len(result[0].split()),
                    "Score": result[1],
                    "Source": "sgrank",
                })

        except StatisticsError:  # issues-8
            pass

        return master

    def process(self,
                input_text: str,
                top_n: int = 15) -> list:

        sw = Stopwatch()

        results = self._process(
            top_n=top_n,
            input_text=input_text)

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "Textacy Keyword Extraction Complete",
                f"\tTotal Time: {str(sw)}",
                f"\tTotal Size: {len(results)}"]))

        return results



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract Ngrams from Input Text using Textacy """


import pandas as pd
from baseblock import BaseObject, Stopwatch
from pandas import DataFrame
from textacy import load_spacy_lang, make_spacy_doc
from textacy.extract.triples import subject_verb_object_triples

from owl_builder.autotaxo.dmo import StopWordFilter


class TextacyTripleExtractor(BaseObject):
    """ Extract Triples from Input Text using Textacy """

    def __init__(self):
        """
        Created:
            15-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/2
        """
        BaseObject.__init__(self, __name__)
        self._has_stopword = StopWordFilter().has_stopword

    def _process(self,
                 input_text: str,
                 drop_determiners: bool,
                 case_sensitive: bool,
                 min_freq: int) -> DataFrame:

        en = load_spacy_lang("en_core_web_sm")

        if not case_sensitive:
            input_text = input_text.lower()

        doc = make_spacy_doc(input_text, lang=en)

        results = list(subject_verb_object_triples(doc))
        print(results)

        # from collections import Counter
        # c = Counter()
        # for result in results:
        #     c.update({result: 1})

        # s = set()
        # for term in c:
        #     frequency = c[term]
        #     if frequency >= min_freq:
        #         s.add(term)

        # return sorted(s)

    def process(self,
                input_text: str,
                min_freq: int = 1,
                filter_stops: bool = True,
                drop_determiners: bool = True,
                case_sensitive: bool = False,
                as_dataframe: bool = False) -> list:
        """ Extract n-Grams from Input Text

        Args:
            input_text (str): input text of any length
            min_freq (int, optional): Number of times a term must repeat. Defaults to 3.
            filter_stops (bool, optional): Filter Stopwords. Defaults to True.
            drop_determiners (bool, optional): drop determiners from the start of noun chunks. Defaults to True.
            case_sensitive (bool, optional): Determine if input text case should be maintained.  Defaults to False
            as_dataframe (bool, optional): Returns the result set as a pandas Dataframe.  Defaults to False

        Returns:
            list: n-Gram results
        """

        # sw = Stopwatch()

        # results = self._process(input_text=input_text,
        #                         drop_determiners=drop_determiners,
        #                         case_sensitive=case_sensitive,
        #                         min_freq=min_freq)

        raise NotImplementedError('needs more work')

        # # the result-set is a list of spacy.tokens.span.Span elements
        # results = [x.text.strip() for x in results]

        # if filter_stops:
        #     results = [x for x in results if not self._has_stopword(x)]

        # # I consider a 'noun chunk' to be 2..* nouns
        # results = [x for x in results if ' ' in x]

        # if self.isEnabledForInfo:
        #     self.logger.info('\n'.join([
        #         "Noun Chunk Extraction Complete",
        #         f"\tTotal Time: {str(sw)}",
        #         f"\tTotal Size: {len(results)}"]))

        # if not as_dataframe:
        #     return results

        # master = []

        # for result in results:
        #     master.append({
        #         "Text": result,
        #         "Size": len(result.split())})

        # return pd.DataFrame(master)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Extract Keyterms from Input Text using Textacy """


from baseblock import BaseObject, Stopwatch, TextUtils
from pandas import DataFrame
from textblob import Word


class FilterKeyterms(BaseObject):
    """ Filter a DataFrame of KeyTerms """

    def __init__(self):
        """ Change Log

        Created:
            18-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/3
        Updated:
            8-Aug-2022
            craigtrim@gmail.com
            *   eliminate subsumed tokens
                https://github.com/craigtrim/buildowl/issues/9
        Updated:
            7-Sept-2022
            craigtrim@gmail.com
            *   eliminate subsumed tokens
                https://github.com/craigtrim/owl-builder/issues/1
        """
        BaseObject.__init__(self, __name__)

    def _process(self,
                 df: DataFrame) -> list:

        if df is None or not len(df):
            return None

        terms = [x.lower() for x in list(df['Term'].unique())]

        subsumed = TextUtils.find_subsumed_tokens(terms)
        terms = [x for x in terms if x not in subsumed]

        # sort by length
        terms = sorted(set(terms), key=len, reverse=True)

        discards = set()

        # discard plurals
        for t1 in terms:
            for t2 in [x for x in terms if x != t1]:
                if f"{t1}s" == t2:
                    discards.add(t2)
                elif f"{t2}s" == t1:
                    discards.add(t1)

        # discard bigrams found in trigrams
        for t1 in [x for x in terms if len(x.split()) == 2]:
            for t2 in [x for x in terms if len(x.split()) == 3]:
                if f"{t1} " in t2 or f" {t1}" in t2:
                    discards.add(t1)

        terms = [x.strip() for x in terms if x not in discards]

        # discard larger length if word similarity value is close
        for t1 in terms:
            for t2 in [x for x in terms if x != t1]:
                if TextUtils.jaccard_similarity(t1, t2) >= 0.85:
                    if len(t1) > len(t2):
                        discards.add(t1)
                    else:
                        discards.add(t2)

        # given a term like 'wi-fi' discard any 'wi' and 'fi' terms
        for tokens in [x.split('-') for x in terms if '-' in x]:
            [discards.add(x) for x in tokens if x in terms]

        if self.isEnabledForDebug:
            self.logger.debug('\n'.join([
                "Tokens Filtering Completed",
                f"\tTotal Discards: {len(discards)}",
                f"\t\t{list(discards)}"]))

        terms = [x.strip() for x in terms if x not in discards]

        def has_bad_char(a_term: str) -> str:
            if '(' in a_term:
                return True
            if ')' in a_term:
                return True
            return False

        terms = [x for x in terms if not has_bad_char(x)]

        # lemmatize individual tokens in each term
        lemmatized_terms = set()
        for term in terms:
            tokens = term.split()
            term = ' '.join([Word(x).lemmatize() for x in tokens]).strip()
            lemmatized_terms.add(term.strip())

        lemmatized_terms = [x.strip() for x in lemmatized_terms]
        return sorted(lemmatized_terms, key=len, reverse=True)

    def process(self,
                df: DataFrame) -> list:
        """ Filter Keyterms

        Args:
            df (DataFrame): incoming pandas dataframe
            Sample Input:
                +----+----------+--------+------------+-----------------+
                |    | Term     |   Size |      Score | Source          |
                |----+----------+--------+------------+-----------------|
                |  0 | meeting  |      1 |   0.134722 | kt.textrank     |
                |  1 | tomorrow |      1 |   0.080897 | kt.textrank     |
                |  2 | pm       |      1 |   0.080897 | kt.textrank     |
                ...
                | 21 | 1 pm     |      2 | nan        | terms.3         |
                +----+----------+--------+------------+-----------------+

        Returns:
            list: the filtered results
        """

        sw = Stopwatch()

        results = self._process(df)

        if self.isEnabledForInfo:
            self.logger.info('\n'.join([
                "Keyword Filtering Complete",
                f"\tTotal Time: {str(sw)}",
                f"\tTotal Size: {len(df)}"]))

        return results



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Orchestrate Taxonomy Generation """


from typing import List
from typing import Optional

from pandas import DataFrame

from baseblock import BaseObject
from baseblock import Enforcer

from owl_builder.autotaxo.svc import ExtractKeyterms
from owl_builder.autotaxo.svc import ExtractKeytermsOpenAI
from owl_builder.autotaxo.svc import FilterKeyterms
from owl_builder.autotaxo.svc import GenerateTaxonomyDataFrame
from owl_builder.autotaxo.svc import GenerateTaxonomyTTL
from owl_builder.autotaxo.dto import load_model


class AutoTaxoOrchestrator(BaseObject):
    """ Orchestrate Taxonomy Generation """

    __model = None

    def __init__(self):
        """ Change Log:

        Created:
            16-Apr-2022
            craigtrim@gmail.com
            *   in pursuit of "Auto Taxonomy Building with Textacy Library #286"
        Updated:
            2-May-2022
            craigtrim@gmail.com
            *   renamed from 'generate-taxonomy'
        Updated:
            18-Jul-2022
            craigtrim@gmail.com
            *   overhaul end-to-end process
                https://github.com/craigtrim/buildowl/issues/3
        Updated:
            16-Aug-2022
            craigtrim@gmail.com
            *   assert return types per
                https://bast-ai.atlassian.net/browse/COR-94?focusedCommentId=10203
        Updated:
            19-Oct-2022
            craigtrim@gmail.com
            *   use lazy loading for model in pursuit of
                https://github.com/craigtrim/climate-mdl-builder/issues/5
        Updated:
            14-Dec-2022
            craigtrim@gmail.com
            *   deprecate 'ttlbuilder' and expose 'build_ttl'
                https://github.com/craigtrim/owl-builder/issues/2
        Updated:
            27-Mar-2023
            craigtrim@gmail.com
            *   add 'keyterms-openai'
                https://github.com/craigtrim/owl-builder/issues/5
        """
        BaseObject.__init__(self, __name__)
        self._keyterms_openai = ExtractKeytermsOpenAI().process

    def _model(self):
        if not self.__model:
            self.__model = load_model()
        return self.__model

    def keyterms_openai(self,
                        input_text: str) -> Optional[List[str]]:
        """ Extract Keyterms using OpenAI

        Args:
            input_text (str): input text of any length or description

        Returns:
            Optional[List[str]]: list of keyterms
        """
        return self._keyterms_openai(input_text)

    def keyterms(self,
                 input_text: str,
                 use_terms: bool = True,
                 use_keyterms: bool = True,
                 use_ngrams: bool = False,
                 use_nounchunks: bool = False) -> Optional[List[str]]:
        """ Generate KeyTerms as a simple list

        Args:
            input_text (str): input text of any length or description
            use_terms (bool, optional). Use Simple Term extraction algorithms. Default is True.
            use_keyterms (bool, optional). Use KeyTerm extraction algorithms. Default is True.
            use_ngrams (bool, optional). Use n-Gram extraction algorithms. Default is False.
            use_nounchunks (bool, optional). Use Noun Chunk extraction algorithms. Default is False.

        Sample Input:
            A local area network (LAN) is a computer network that interconnects computers within a limited area such as a residence, school, laboratory, university campus or office building.
            By contrast, a wide area network (WAN) not only covers a larger geographic distance, but also generally involves leased telecommunication circuits.
            Ethernet and Wi-Fi are the two most common technologies in use for local area networks.
            Historical network technologies include ARCNET, Token Ring, and AppleTalk.

        Sample Output:
            [   'leased telecommunication circuit',
                'historical network technology',
                'large geographic distance',
                'interconnects computer',
                'local area network',
                'university campus',
                'common technology',
                'wide area network',
                'computer network',
                'office building',
                'include arcnet',
                'limited area',
                'token ring'
            ]

        Returns:
            Optional[List[str]]: list of keyterms
        """
        svc = ExtractKeyterms(self._model())

        if self.isEnabledForDebug:
            Enforcer.is_str(input_text)

        df_keyterms = svc.process(input_text,
                                  use_terms=use_terms,
                                  use_keyterms=use_keyterms,
                                  use_ngrams=use_ngrams,
                                  use_nounchunks=use_nounchunks)

        if self.isEnabledForDebug:
            assert type(df_keyterms) == DataFrame

        keyterms = FilterKeyterms().process(df_keyterms)

        if self.isEnabledForDebug:
            Enforcer.is_optional_list(keyterms)

        return keyterms

    def dataframe(self,
                  input_text: str) -> DataFrame:
        """ Generate KeyTerms as a Pandas DataFrame

        Args:
            input_text (str): input text of any length or description

        Sample Input:
            A local area network (LAN) is a computer network that interconnects computers within a limited area such as a residence, school, laboratory, university campus or office building.
            By contrast, a wide area network (WAN) not only covers a larger geographic distance, but also generally involves leased telecommunication circuits.
            Ethernet and Wi-Fi are the two most common technologies in use for local area networks.
            Historical network technologies include ARCNET, Token Ring, and AppleTalk.

        Sample Output:
            +----+---------------------------+----------------------------------+--------------+
            |    | Parent                    | Child                            | Confidence   |
            |----+---------------------------+----------------------------------+--------------|
            |  0 | circuit                   | telecommunication circuit        | e            |
            |  1 | telecommunication circuit | leased telecommunication circuit | e            |
            |  2 | technology                | network technology               | i            |
            |  3 | network technology        | historical network technology    | i            |
            |  4 | distance                  | geographic distance              | a            |
            |  5 | geographic distance       | large geographic distance        | a            |
            |  6 | computer                  | interconnects computer           | n            |
            |  7 | network                   | area network                     | o            |
            |  8 | area network              | local area network               | o            |
            |  9 | network                   | area network                     | i            |
            | 10 | area network              | wide area network                | i            |
            | 11 | technology                | common technology                | o            |
            | 12 | campus                    | university campus                | n            |
            | 13 | network                   | computer network                 | o            |
            | 14 | building                  | office building                  | f            |
            | 15 | arcnet                    | include arcnet                   | n            |
            | 16 | area                      | limited area                     | i            |
            | 17 | ring                      | token ring                       | o            |
            +----+---------------------------+----------------------------------+--------------+
        Returns:
            DataFrame: list of keyterms
        """

        keyterms = self.keyterms(input_text)
        return GenerateTaxonomyDataFrame().process(keyterms)

    def build_ttl(self,
                  input: str or DataFrame) -> Optional[List[str]]:
        """ Generate TTL Results for augmenatation of OWL model

        Args:
            input (str or DataFrame): either input text or a DataFrame

                input_text:  any input text string of any length
                    Purpose:
                        If the consumer is sending plain-text input it means they want this microservice to create the taxonomy
                        prior to generate the TTL code
                    Sample Input:
                        A local area network (LAN) is a computer network that interconnects computers within a limited area.

                DataFrame: a Dataframe with Parent/Child taxonomy columns.
                    Purpose:
                        If the consumer is sending a Datafraame of this format, it means they want the TTL generator to use
                        their own taxonomy, rather than creating one on-the-fly
                    Sample Input:
                        +----+---------------------------+----------------------------------+
                        |    | Parent                    | Child                            |
                        |----+---------------------------+----------------------------------+
                        |  0 | circuit                   | telecommunication circuit        |
                        |  1 | telecommunication circuit | leased telecommunication circuit |
                        |  2 | technology                | network technology               |
                        +----+---------------------------+----------------------------------+

        Returns:
            Optional[List[str]]: TTL results for OWL model
                Sample Output:
                    ###  http://graffl.ai/pathology#telecommunication_circuit
                                :telecommunication_circuit rdf:type owl:Class ;
                                rdfs:label "Telecommunication Circuit" ;
                                rdfs:subClassOf :circuit .
                    ###  http://graffl.ai/pathology#circuit
                                :circuit rdf:type owl:Class ;
                                rdfs:label "Circuit" .
                    ###  http://graffl.ai/pathology#leased_telecommunication_circuit
                                :leased_telecommunication_circuit rdf:type owl:Class ;
                                rdfs:label "Leased Telecommunication Circuit" ;
                                rdfs:subClassOf :telecommunication_circuit .
                    ...
                    ###  http://graffl.ai/pathology#token_ring
                                :token_ring rdf:type owl:Class ;
                                rdfs:label "Token Ring" ;
                                rdfs:subClassOf :ring .
                    ###  http://graffl.ai/pathology#ring
                                :ring rdf:type owl:Class ;
                                rdfs:label "Ring" .
        """

        def get_taxonomy_df() -> DataFrame:
            type_input = type(input)
            if type_input == str:
                return self.dataframe(input)
            elif type(input) == DataFrame:
                if not set(['Parent', 'Child']).issubset(set(input.columns)):
                    raise ValueError("Unexpected DataFrame")
                return input
            raise TypeError(type_input)

        ttl_generator = GenerateTaxonomyTTL().process
        return ttl_generator(get_taxonomy_df())

    # TODO: Deprecated on 14-DEC-2022
    def ttlresults(self, input_text: str) -> list or None:
        self.logger.warn(
            "\n\n!!!This Method is Deprecated -- use 'build_ttl' instead\n\n!!!")
        df_taxo = self.dataframe(input_text)
        return GenerateTaxonomyTTL().process(df_taxo)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Initialize a simple Disk-Backed Cache for OpenAI """


import os

from baseblock import BaseObject, EnvIO, FileIO


class OpenAICache(BaseObject):
    """ Initialize and Operate a simple Disk-Backed Cache for OpenAI """

    def __init__(self):
        """
        Created:
            20-Jul-2022
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/6
        """
        BaseObject.__init__(self, __name__)
        self._cache_path = EnvIO.str_or_default(
            'AUTOSYN_CACHE_PATH', 'resources/cache/syns')
        FileIO.exists_or_create(self._cache_path)

    def _file_path(self,
                   file_name: str) -> str:
        def base() -> str:
            if len(file_name) >= 3:
                return os.path.join(self._cache_path,
                                    file_name[0],
                                    file_name[1],
                                    file_name[2])
            if len(file_name) >= 2:
                return os.path.join(self._cache_path,
                                    file_name[0],
                                    file_name[1])
            if len(file_name) >= 1:
                return os.path.join(self._cache_path,
                                    file_name[0])

        base_dir = base()

        FileIO.exists_or_create(base_dir)
        return os.path.join(base_dir, f"{file_name}.json")

    def exists(self,
               file_name: str) -> bool:
        """ Check if File exists in Cache

        Args:
            file_name (str): the file name

        Returns:
            bool: True if the file exists (this does not indicate if the file has content; only if the file exists)
        """
        try:
            file_path = self._file_path(file_name)
            return FileIO.exists(file_path)
        except TypeError as e:
            self.logger.error('\n'.join([
                "File Path Error",
                f"\tFile Name: {file_name}",
                f"\tError: {e}"]))
            return False

    def read(self,
             file_name: str) -> dict or list or None:
        """ Read Data from Cache

        Args:
            file_name (str): the file name

        Returns:
            dict or list or None: the JSON object (if any)
        """
        file_path = self._file_path(file_name)
        if FileIO.exists(file_path):
            return FileIO.read_json(file_path)

    def write(self,
              data: dict or list,
              file_name: str) -> None:
        """ Write Data to Cache

        Args:
            data (dict or list): the JSON object
            file_name (str): the name of the file to write
        """
        FileIO.write_json(data=data, file_path=self._file_path(file_name))



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Orchestrate Taxonomy Generation """


from baseblock import BaseObject
from rdflib import Graph

from owl_builder.buildr.dmo import OwlGraphConnector
from owl_builder.buildr.svc import AddGraphNodes, GenerateOwlModel, OpenAsGraph


class OwlBuilderAPI(BaseObject):
    """ Orchestrate Taxonomy Generation """

    def __init__(self):
        """ Change Log:

        Created:
            21-Jul-20922
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/7
        """
        BaseObject.__init__(self, __name__)

    def generate(self,
                 model_name: str,
                 model_path: str,
                 model_author: str = None) -> str:
        """ Generate an OWL model

        Args:
            model_name (str): the name of the model
                this will form part of the namespace URI
            model_path (str): the directory the model will be persisted to
            model_author (str, optional): the model author. Defaults to None.

        Returns:
            str: the absolute path to the OWL model
        """
        return GenerateOwlModel().process(
            model_name=model_name,
            model_path=model_path,
            model_author=model_author)

    def add_entities(self,
                     model_file_name: str,
                     ttl_entities: list) -> str:
        """ Add new Entities in TTL (Turtle) format to the Model

        Args:
            model_file_name (str): the path to the existing model
            ttl_entities (list): the list of entities

        Returns:
            str: the new version
        """
        return AddGraphNodes().process(
            ttl_entities=ttl_entities,
            model_file_name=model_file_name)

    def to_graph(self,
                 model_file_name: str) -> Graph:
        """ Open a file-based model as an rdflib Graph

        Args:
            model_file_name (str): the path to the existing model

        Returns:
            Graph: an instantiation of an rdflib Graph object
        """
        return OpenAsGraph().process(model_file_name)



#!/usr/bin/env python
# -*- coding: UTF-8 -*-
""" Add Graph Nodes to the OWL Model """


import os
from datetime import datetime

import pandas as pd
from baseblock import BaseObject, EnvIO, FileIO
from pandas import DataFrame
from rdflib import Graph, Namespace

from owl_builder.autorels.svc import (FindImpliesRelationships,
                                      FindRequiresRelationships,
                                      GenerateRelationshipsTTL)


class AddGraphNodes(BaseObject):
    """ Add Graph Nodes to the OWL Model """

    __authorship = "Generated on TIMESTAMP by AUTHOR"

    def __init__(self):
        """ Change Log:

        Created:
            21-Jul-20922
            craigtrim@gmail.com
            *   https://github.com/craigtrim/buildowl/issues/7
        """
        BaseObject.__init__(self, __name__)

    @staticmethod
    def _bump_version(lines: list) -> str:
        """ Increment the Minor Version Indicator
            all other increments must be performed by the author, manually

        Args:
            lines (list): all the graph lines

        Returns:
            str: the updated version only
        """

        def get_new_version(version_line: str) -> str:
            """ Get the new Version String
            Args:
                version_line (str): the incoming RDF line with the version
                    "owl:versionIRI <http://graffl.ai/template/0.1.0> ."

            Returns:
                str: the updated version only
            """
            current_version = version_line.split(
                "graffl.ai/")[-1].split("/")[-1].split('>')[0]
            tokens = str(current_version).split('.')
            return f"{tokens[0]}.{tokens[1]}.{int(tokens[2]) + 1}"

        for i in range(len(lines)):
            if "owl:versionIRI" in lines[i]:
                return get_new_version(lines[i])

        raise ValueError("No Version Defined")

    @staticmethod
    def _starting_line(lines: list) -> int:
        """ Find the Line to add new entities

        This isn't a big deal; the OWL is self-organizing when manually edited

        Args:
            lines (list): all the graph lines

        Returns:
            int: the preferred position to add new entities
        """
        for i in range(len(lines)):
            if lines[i].strip().startswith("###  Generated by the OWL API"):
                return i
            return len(lines) - 1

    def process(self,
                model_file_name: str,
                ttl_entities: list) -> str:
        """ Add new Entities in TTL (Turtle) format to the Model

        Args:
            model_file_name (str): the path to the existing model
            ttl_entities (list): the list of entities

        Returns:
            str: the new version
        """

        FileIO.exists_or_error(model_file_name)
        lines = FileIO.read_lines(model_file_name)

        version = self._bump_version(lines)
        start_on_line = self._starting_line(lines)

        normalized = []

        for i in range(len(lines)):

            if "owl:versionIRI" in lines[i]:
                vline = f"owl:versionIRI <http://graffl.ai/template/{version}> ."
                normalized.append(vline)
                continue

            if i == start_on_line:
                [normalized.append(x) for x in ttl_entities]

            normalized.append(lines[i])

        FileIO.write_lines(normalized, model_file_name)
        return version



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['owl_builder',
 'owl_builder.autorels',
 'owl_builder.autorels.bp',
 'owl_builder.autorels.svc',
 'owl_builder.autorels.tests',
 'owl_builder.autosyns',
 'owl_builder.autosyns.bp',
 'owl_builder.autosyns.dmo',
 'owl_builder.autosyns.dto',
 'owl_builder.autosyns.svc',
 'owl_builder.autosyns.tests',
 'owl_builder.autotaxo',
 'owl_builder.autotaxo.bp',
 'owl_builder.autotaxo.dmo',
 'owl_builder.autotaxo.dto',
 'owl_builder.autotaxo.svc',
 'owl_builder.autotaxo.tests',
 'owl_builder.buildr',
 'owl_builder.buildr.bp',
 'owl_builder.buildr.dmo',
 'owl_builder.buildr.svc',
 'owl_builder.buildr.tests']

package_data = \
{'': ['*']}

install_requires = \
['baseblock',
 'networkx==2.8.8',
 'nltk==3.8.1',
 'openai>=0.27.0,<0.28.0',
 'openai_helper',
 'pandas>=1.4.0,<2.0.0',
 'rdflib>=6.1.1,<7.0.0',
 'regex==2022.7.9',
 'scipy==1.9.2',
 'spacy==3.5.0',
 'tabulate',
 'textacy==0.12.0',
 'textblob>=0.17.1,<0.18.0']

setup_kwargs = {
    'name': 'owl-builder',
    'version': '0.1.19',
    'description': 'Tools for Automating the Construction of an Ontology (OWL)',
    'long_description': '# Ontology Builder (owl-builder)\n\n##\n\n## Key Term Extraction\n```python\nfrom owl_builder import keyterms\n\ninput_text = """\nA local area network (LAN) is a computer network that interconnects computers within a limited area such as a residence, school, laboratory, university campus or office building.\n\nBy contrast, a wide area network (WAN) not only covers a larger geographic distance, but also generally involves leased telecommunication circuits.\n\nEthernet and Wi-Fi are the two most common technologies in use for local area networks.\n\nHistorical network technologies include ARCNET, Token Ring, and AppleTalk.\n"""\n\nresults = keyterms(\n    input_text=input_text,\n    use_terms=True,\n    use_keyterms=True,\n    use_ngrams=False,\n    use_nounchunks=False)\n```\n\nThe results are\n```json\n[\n   "leased telecommunication circuit",\n   "historical network technology",\n   "large geographic distance",\n   "interconnects computer",\n   "local area network",\n   "university campus",\n   "common technology",\n   "wide area network",\n   "computer network",\n   "office building",\n   "include arcnet",\n   "limited area",\n   "token ring"\n]\n```\n\n## TTL Generation\n```python\nfrom owl_builder import build_ttl\n\nresults = build_ttl("He has aims to make Detroit a leader in green energy.")\n```\n\nThe result is\n```ttl\n###  http://graffl.ai/pathology#green_energy\n        :green_energy rdf:type owl:Class ;\n        rdfs:label "Green Energy" ;\n        rdfs:subClassOf :energy .\n###  http://graffl.ai/pathology#energy\n        :energy rdf:type owl:Class ;\n        rdfs:label "Energy" .\n```\n\nYou can also supply your own taxonomy like this:\n```python\nimport pandas as pd\n\nresults = build_ttl(pd.DataFrame([\n    {"Parent": "Alpha", "Child": "Alpha Beta"},\n    {"Parent": "Alpha Beta", "Child": "Alpha Beta Gamma"},\n    {"Parent": "Gamma", "Child": "Gamma Delta"},\n]))\n```\n\nThe result is\n```ttl\n###  http://graffl.ai/pathology#alpha_beta\n        :alpha_beta rdf:type owl:Class ;\n        rdfs:label "Alpha Beta" ;\n        rdfs:subClassOf :alpha .\n###  http://graffl.ai/pathology#alpha\n            :alpha rdf:type owl:Class ;\n            rdfs:label "Alpha" .\n###  http://graffl.ai/pathology#alpha_beta_gamma\n            :alpha_beta_gamma rdf:type owl:Class ;\n            rdfs:label "Alpha Beta Gamma" ;\n            rdfs:subClassOf :alpha_beta .\n###  http://graffl.ai/pathology#alpha_beta\n            :alpha_beta rdf:type owl:Class ;\n            rdfs:label "Alpha Beta" .\n###  http://graffl.ai/pathology#gamma_delta\n            :gamma_delta rdf:type owl:Class ;\n            rdfs:label "Gamma Delta" ;\n            rdfs:subClassOf :gamma .\n###  http://graffl.ai/pathology#gamma\n            :gamma rdf:type owl:Class ;\n            rdfs:label "Gamma" .\n```\n',
    'author': 'Craig Trim',
    'author_email': 'craigtrim@gmail.com',
    'maintainer': 'Craig Trim',
    'maintainer_email': 'craigtrim@gmail.com',
    'url': 'https://github.com/craigtrim/owl-builder',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8.5,<4.0.0',
}


setup(**setup_kwargs)



from baseblock import Enforcer

from owl_builder.autotaxo.bp import AutoTaxoOrchestrator

input_text = """
    A local area network (LAN) is a computer network that interconnects computers within a limited area such as a residence, school, laboratory, university campus or office building.
    By contrast, a wide area network (WAN) not only covers a larger geographic distance, but also generally involves leased telecommunication circuits.
    Ethernet and Wi-Fi are the two most common technologies in use for local area networks.
    Historical network technologies include ARCNET, Token Ring, and AppleTalk.
""".strip()


def test_bp():
    bp = AutoTaxoOrchestrator()
    assert bp

    results = bp.ttlresults(input_text)
    Enforcer.is_list_of_str(results)
